import bpy
import bpy.utils.previews
import random
import os
from datetime import datetime
from bpy.app.handlers import persistent
from bpy.app.translations import pgettext_iface as iface_

bl_info = {
    "name": "Custom Output Path",
    "author": "Your Name",
    "version": (1, 0),
    "blender": (2, 80, 0),
    "location": "Node Editor > Node > Custom Output Path",
    "description": "Customize the output file paths of File Output nodes",
    "warning": "",
    "doc_url": "",
    "category": "Node",
}

_icons = None
cached_random_numbers = {}

# Translation context for UI
translation_dict = {
    "en_US": {
        ("*", "Active Camera Info:"): "Active Camera Info:",
        ("*", "Name"): "Name",
        ("*", "Type"): "Type",
        ("*", "Scale"): "Scale",
        ("*", "Focal Length"): "Focal Length",
        ("*", "No active camera"): "No active camera",
        ("*", "Orthographic"): "Orthographic",
        ("*", "Perspective"): "Perspective",
        ("*", "Time Options:"): "Time Options:",
        ("*", "Random Options:"): "Random Options:",
        ("*", "Sequence Options:"): "Sequence Options:",
        ("*", "Suffix Options:"): "Suffix Options:",
        ("*", "Auto"): "Auto",
        ("*", "Filename Options:"): "Filename Options:",
        ("*", "Use Interface Name"): "Use Interface Name",
    },
    "zh_CN": {
        ("*", "Active Camera Info:"): "当前相机信息：",
        ("*", "Name"): "名称",
        ("*", "Type"): "类型",
        ("*", "Scale"): "缩放",
        ("*", "Focal Length"): "焦距",
        ("*", "No active camera"): "无活动相机",
        ("*", "Orthographic"): "正交",
        ("*", "Perspective"): "透视",
        ("*", "Time Options:"): "时间选项：",
        ("*", "Random Options:"): "随机选项：",
        ("*", "Sequence Options:"): "序列选项：",
        ("*", "Suffix Options:"): "后缀选项：",
        ("*", "Auto"): "自动",
        ("*", "Filename Options:"): "文件名选项：",
        ("*", "Use Interface Name"): "使用接口名称",
    },
}

def sanitize_path(path):
    invalid_chars = '<>:"/\\|?*'
    for char in invalid_chars:
        path = path.replace(char, '_')
    path = path.replace("\\", "/")
    return path[:255] if len(path) > 255 else path

def format_time(current_time, settings):
    date_parts = []
    time_parts = []
    if settings.use_year: date_parts.append(current_time[:4])
    if settings.use_month: date_parts.append(current_time[5:7])
    if settings.use_day: date_parts.append(current_time[8:10])
    if settings.use_hour: time_parts.append(current_time[11:13])
    if settings.use_minute: time_parts.append(current_time[14:16])
    if settings.use_second: time_parts.append(current_time[17:19])
    date_string = "-".join(date_parts)
    time_string = ".".join(time_parts)
    return f"({date_string} {time_string})_" if date_string or time_string else ""

def trace_input_source(input_socket):
    """追踪输入接口的实际来源，优先使用节点组输出接口名称"""
    if not input_socket.links:
        return None, None
    
    current_socket = input_socket.links[0].from_socket
    current_node = input_socket.links[0].from_node
    
    # 如果直接连接的是节点组，使用其输出接口名称
    if current_node.type == 'GROUP':
        return current_node, current_socket
    
    # 追踪通过转接节点
    while current_node and current_node.type in {'REROUTE', 'GROUP_INPUT', 'GROUP_OUTPUT'}:
        if not current_node.inputs or not current_node.inputs[0].links:
            break
        next_socket = current_node.inputs[0].links[0].from_socket
        next_node = current_node.inputs[0].links[0].from_node
        # 如果遇到节点组，停止追踪并使用其输出接口名称
        if next_node and next_node.type == 'GROUP':
            return next_node, next_socket
        current_socket = next_socket
        current_node = next_node
    
    return current_node, current_socket

def generate_new_path(settings, filename, current_time, random_number, current_frame, sequence_number, sequence_padding, i):
    path_parts = []
    if settings.use_suffix_rule and settings.custom_prefix:
        path_parts.append(settings.custom_prefix.rstrip('_') + "_")
    if settings.use_filename_rule:
        if settings.use_interface_name and hasattr(bpy.context, 'active_node') and bpy.context.active_node:
            node = bpy.context.active_node
            if node.type == 'OUTPUT_FILE' and len(node.inputs) > i and node.inputs[i].links:
                source_node, source_socket = trace_input_source(node.inputs[i])
                if source_node and source_socket:
                    # 使用实际源节点的接口名称
                    interface_name = source_socket.name
                    path_parts.append(interface_name + "_")
                else:
                    path_parts.append(filename + "_")
            else:
                path_parts.append(filename + "_")
        else:
            path_parts.append(filename + "_")
    if settings.use_time_rule:
        path_parts.append(format_time(current_time, settings))
    if settings.use_random_rule and random_number is not None:
        path_parts.append(f"{random_number}_")
    if settings.use_frame_rule:
        path_parts.append(f"f{current_frame}_")
    if settings.use_camera_rule and bpy.context.scene.camera:
        path_parts.append(f"{bpy.context.scene.camera.name}_")
    if settings.use_sequence_rule:
        path_parts.append(f"{str(sequence_number).zfill(sequence_padding)}_")
    else:
        path_parts.append(f"{i + 1:02d}_")
    new_path = "".join(path_parts).rstrip("_")
    if settings.use_suffix_rule and settings.custom_suffix:
        new_path += f"_{settings.custom_suffix.lstrip('_')}"
    return sanitize_path(new_path)

def rename_output_file_paths(node, settings, current_frame=None):
    try:
        if not node or not hasattr(node, 'file_slots'):
            return
        blend_filepath = bpy.data.filepath or "Unsaved_File"
        filename = os.path.splitext(os.path.basename(blend_filepath))[0]
        current_time = settings.cached_current_time or datetime.now().strftime("%Y_%m_%d_%H_%M_%S")
        sequence_number = max(settings.sequence_start, 1)
        sequence_padding = max(settings.sequence_padding, 1)
        current_frame = current_frame or bpy.context.scene.frame_current

        for i, file_slot in enumerate(node.file_slots):
            random_number = None
            key = (node.as_pointer(), i)
            if settings.use_random_rule:
                if settings.auto_random:
                    if key not in cached_random_numbers:
                        random_number = random.randint(10001, 99999)
                        cached_random_numbers[key] = random_number
                    else:
                        random_number = cached_random_numbers[key]
                else:
                    if key in cached_random_numbers:
                        random_number = cached_random_numbers[key]
                    else:
                        random_number = random.randint(10001, 99999)
                        cached_random_numbers[key] = random_number
            new_path = generate_new_path(settings, filename, current_time, random_number,
                                         current_frame, sequence_number, sequence_padding, i)
            file_slot.path = new_path
            if settings.use_sequence_rule:
                sequence_number += 1
    except Exception as e:
        print(f"Error in rename_output_file_paths: {e}")

def sync_node_selection(settings, node_tree):
    try:
        existing_selections = {}
        for item in settings.node_selection:
            node = node_tree.nodes.get(item.node_name)
            if node:
                existing_selections[node.as_pointer()] = (item.node_name, item.select)
        settings.node_selection.clear()
        for node in node_tree.nodes:
            if node.type == 'OUTPUT_FILE':
                item = settings.node_selection.add()
                item.node_name = node.name
                _, select_state = existing_selections.get(node.as_pointer(), (node.name, False))
                item.select = select_state
    except Exception as e:
        print(f"Error in sync_node_selection: {e}")

def naming_rules_update_callback(self, context):
    update_naming_rules(self, context)

def update_naming_rules(settings, context):
    if getattr(settings, 'updating', False):
        return
    settings.updating = True
    try:
        if not settings.use_frame_rule:
            pass
        if not settings.use_random_rule and settings.auto_random:
            settings.auto_random = False
        if not settings.use_sequence_rule:
            if settings.sequence_start != 1:
                settings.sequence_start = 1
            if settings.sequence_padding != 3:
                settings.sequence_padding = 3
        if not settings.use_suffix_rule:
            if settings.custom_prefix:
                settings.custom_prefix = ""
            if settings.custom_suffix:
                settings.custom_suffix = ""
        if not settings.use_time_rule:
            if any([settings.use_year, settings.use_month, settings.use_day, settings.use_hour, settings.use_minute, settings.use_second]):
                settings.use_year = True
                settings.use_month = True
                settings.use_day = True
                settings.use_hour = True
                settings.use_minute = True
                settings.use_second = True

        # 获取所有可能的节点树
        node_trees = []
        if hasattr(context.scene, 'node_tree') and context.scene.node_tree:
            node_trees.append(context.scene.node_tree)
        for mat in bpy.data.materials:
            if mat.use_nodes and mat.node_tree:
                node_trees.append(mat.node_tree)
        for world in bpy.data.worlds:
            if world.use_nodes and world.node_tree:
                node_trees.append(world.node_tree)

        if settings.use_multiple_nodes:
            # 遍历所有节点树中的文件输出节点
            for node_tree in node_trees:
                for node in node_tree.nodes:
                    if node.type == 'OUTPUT_FILE':
                        # 检查节点是否在选择列表中，如果不在则添加
                        node_found = False
                        for item in settings.node_selection:
                            if item.node_name == node.name:
                                node_found = True
                                if item.select:
                                    rename_output_file_paths(node, settings)
                                break
                        if not node_found:
                            item = settings.node_selection.add()
                            item.node_name = node.name
                            item.select = True
                            rename_output_file_paths(node, settings)
        else:
            # 处理活动节点
            if context.active_node and context.active_node.type == 'OUTPUT_FILE':
                rename_output_file_paths(context.active_node, settings)
    except Exception as e:
        print(f"Error in update_naming_rules: {e}")
    finally:
        settings.updating = False

def update_node_selection_item_callback(self, context):
    """当节点选择状态改变时的回调函数"""
    settings = context.scene.naming_rules_settings
    if not settings.use_multiple_nodes:
        return

    # 获取所有节点树
    node_trees = []
    if hasattr(context.scene, 'node_tree') and context.scene.node_tree:
        node_trees.append(context.scene.node_tree)
    for mat in bpy.data.materials:
        if mat.use_nodes and mat.node_tree:
            node_trees.append(mat.node_tree)
    for world in bpy.data.worlds:
        if world.use_nodes and world.node_tree:
            node_trees.append(world.node_tree)

    # 在所有节点树中查找对应的节点并应用命名规则
    for node_tree in node_trees:
        node = node_tree.nodes.get(self.node_name)
        if node and node.type == 'OUTPUT_FILE':
            if self.select:  # 如果节点被选中
                rename_output_file_paths(node, settings)
            break

class NodeSelectionItem(bpy.types.PropertyGroup):
    node_name: bpy.props.StringProperty()
    select: bpy.props.BoolProperty(name="Select", default=False, update=update_node_selection_item_callback)

class NODE_OT_RenameNode(bpy.types.Operator):
    bl_idname = "node.rename_node"
    bl_label = "Rename Node"
    bl_description = "Rename the selected File Output node"
    bl_options = {'REGISTER', 'UNDO'}

    node_name: bpy.props.StringProperty()
    new_name: bpy.props.StringProperty(name="New Name")

    def execute(self, context):
        try:
            node_tree = context.scene.node_tree
            node = node_tree.nodes.get(self.node_name)
            if node and self.new_name:
                if self.new_name in node_tree.nodes:
                    self.report({'ERROR'}, f"A node with name '{self.new_name}' already exists.")
                    return {'CANCELLED'}
                node.name = node.label = self.new_name
                settings = context.scene.naming_rules_settings
                for item in settings.node_selection:
                    if item.node_name == self.node_name:
                        item.node_name = self.new_name
                        break
                return {'FINISHED'}
            else:
                self.report({'ERROR'}, "Invalid node or name.")
                return {'CANCELLED'}
        except Exception as e:
            self.report({'ERROR'}, f"Error renaming node: {e}")
            return {'CANCELLED'}

    def invoke(self, context, event):
        return context.window_manager.invoke_props_dialog(self)

class NODE_OT_RenameCamera(bpy.types.Operator):
    bl_idname = "node.rename_camera"
    bl_label = "Rename Camera"
    bl_description = "Rename the active camera"
    bl_options = {'REGISTER', 'UNDO'}

    new_name: bpy.props.StringProperty(name="New Name")

    def execute(self, context):
        if not context.scene.camera:
            return {'CANCELLED'}
        
        try:
            old_name = context.scene.camera.name
            # 同时重命名相机对象和数据
            context.scene.camera.name = self.new_name
            context.scene.camera.data.name = self.new_name + "_data"
            
            # 相机重命名后，更新所有使用了相机名称规则的节点
            settings = context.scene.naming_rules_settings
            if settings.use_camera_rule:
                if settings.use_multiple_nodes:
                    for item in settings.node_selection:
                        if item.select:
                            node = context.scene.node_tree.nodes.get(item.node_name)
                            if node and node.type == 'OUTPUT_FILE':
                                rename_output_file_paths(node, settings)
                else:
                    node = context.scene.node_tree.nodes.active
                    if node and node.type == 'OUTPUT_FILE':
                        rename_output_file_paths(node, settings)
            
            return {'FINISHED'}
        except Exception as e:
            self.report({'ERROR'}, f"Error renaming camera: {e}")
            return {'CANCELLED'}

    def invoke(self, context, event):
        if not context.scene.camera:
            return {'CANCELLED'}
        self.new_name = context.scene.camera.name
        return context.window_manager.invoke_props_dialog(self)

class NamingRulesSettings(bpy.types.PropertyGroup):
    updating: bpy.props.BoolProperty(default=False)
    is_rendering: bpy.props.BoolProperty(default=False)
    use_filename_rule: bpy.props.BoolProperty(
        name="Use Filename Rule",
        default=False,
        description="Use the blend file name as part of the output file name",
        update=naming_rules_update_callback
    )
    use_interface_name: bpy.props.BoolProperty(
        name="Use Interface Name",
        default=False,
        description="Use the connected node group interface name",
        update=naming_rules_update_callback
    )
    use_time_rule: bpy.props.BoolProperty(
        name="Use Time Rule",
        default=False,
        description="Include the current time in the output file name",
        update=naming_rules_update_callback
    )
    use_random_rule: bpy.props.BoolProperty(
        name="Use Random Rule",
        default=False,
        description="Add a random number to the output file name",
        update=naming_rules_update_callback
    )
    auto_random: bpy.props.BoolProperty(
        name="Auto",
        default=False,
        description="Automatically generate new random number during rendering",
        update=naming_rules_update_callback
    )
    use_sequence_rule: bpy.props.BoolProperty(
        name="Use Sequence Rule",
        default=False,
        description="Add a sequence number to the output file name",
        update=naming_rules_update_callback
    )
    use_frame_rule: bpy.props.BoolProperty(
        name="Use Frame Rule",
        default=False,
        description="Add the current frame number to the output file name",
        update=naming_rules_update_callback
    )
    use_suffix_rule: bpy.props.BoolProperty(
        name="Use Suffix",
        default=False,
        description="Enable custom prefix and suffix for the output file name",
        update=naming_rules_update_callback
    )
    use_camera_rule: bpy.props.BoolProperty(
        name="Use Camera Rule",
        default=False,
        description="Add the camera name to the output file name",
        update=naming_rules_update_callback
    )
    sequence_start: bpy.props.IntProperty(
        name="Sequence Start",
        default=1,
        min=1,
        description="The starting number for the sequence rule",
        update=naming_rules_update_callback
    )
    sequence_padding: bpy.props.IntProperty(
        name="Sequence Padding",
        default=3,
        min=1,
        description="The number of digits for the sequence number (e.g., 001, 002)",
        update=naming_rules_update_callback
    )
    custom_prefix: bpy.props.StringProperty(
        name="Custom Prefix",
        default="",
        description="A custom prefix to be added before the output file name",
        update=naming_rules_update_callback
    )
    custom_suffix: bpy.props.StringProperty(
        name="Custom Suffix",
        default="",
        description="A custom suffix to be added after the output file name",
        update=naming_rules_update_callback
    )
    use_year: bpy.props.BoolProperty(
        name="Year",
        default=True,
        description="Include the current year in the output file name",
        update=naming_rules_update_callback
    )
    use_month: bpy.props.BoolProperty(
        name="Month",
        default=True,
        description="Include the current month in the output file name",
        update=naming_rules_update_callback
    )
    use_day: bpy.props.BoolProperty(
        name="Day",
        default=True,
        description="Include the current day in the output file name",
        update=naming_rules_update_callback
    )
    use_hour: bpy.props.BoolProperty(
        name="Hour",
        default=True,
        description="Include the current hour in the output file name",
        update=naming_rules_update_callback
    )
    use_minute: bpy.props.BoolProperty(
        name="Minute",
        default=True,
        description="Include the current minute in the output file name",
        update=naming_rules_update_callback
    )
    use_second: bpy.props.BoolProperty(
        name="Second",
        default=True,
        description="Include the current second in the output file name",
        update=naming_rules_update_callback
    )
    use_multiple_nodes: bpy.props.BoolProperty(
        name="Use Multiple Nodes",
        default=False,
        description="Apply naming rules to multiple File Output nodes",
        update=naming_rules_update_callback
    )
    node_selection: bpy.props.CollectionProperty(type=NodeSelectionItem)
    node_selection_index: bpy.props.IntProperty()
    cached_current_time: bpy.props.StringProperty(default="")
    last_frame: bpy.props.IntProperty(default=-1)
    show_camera_info: bpy.props.BoolProperty(
        name="Show Camera Info",
        default=True,
        description="Show or hide camera information"
    )
    show_time_options: bpy.props.BoolProperty(
        name="Show Time Options",
        default=True,
        description="Show or hide time options"
    )
    show_random_options: bpy.props.BoolProperty(
        name="Show Random Options",
        default=True,
        description="Show or hide random options"
    )
    show_sequence_options: bpy.props.BoolProperty(
        name="Show Sequence Options",
        default=True,
        description="Show or hide sequence options"
    )
    show_suffix_options: bpy.props.BoolProperty(
        name="Show Suffix Options",
        default=True,
        description="Show or hide suffix options"
    )
    show_filename_options: bpy.props.BoolProperty(
        name="Show Filename Options",
        default=True,
        description="Show or hide filename options"
    )
    show_multiple_nodes_options: bpy.props.BoolProperty(
        name="Show Multiple Nodes Options",
        default=True,
        description="Show or hide multiple nodes options"
    )

class NODE_PT_CustomOutputPath(bpy.types.Panel):
    bl_label = "Custom Output Path"
    bl_idname = "NODE_PT_custom_output_path"
    bl_space_type = 'NODE_EDITOR'
    bl_region_type = 'UI'
    bl_category = "Node"
    bl_context = "node"

    @classmethod
    def poll(cls, context):
        active_node = context.scene.node_tree.nodes.active
        return (context.space_data.tree_type == 'CompositorNodeTree' and
                context.scene.use_nodes and
                active_node is not None and 
                active_node.type == 'OUTPUT_FILE')

    def draw(self, context):
        layout = self.layout
        settings = context.scene.naming_rules_settings

        layout.enabled = not settings.is_rendering

        row = layout.row(align=True)
        row.prop(settings, "use_filename_rule", text="Filename", toggle=True)
        row.prop(settings, "use_time_rule", text="Time", toggle=True)
        row.prop(settings, "use_random_rule", text="Random", toggle=True)
        row.prop(settings, "use_frame_rule", text="Frame", toggle=True)
        row.prop(settings, "use_sequence_rule", text="Sequence", toggle=True)
        row.prop(settings, "use_suffix_rule", text="Suffix", toggle=True)
        row.prop(settings, "use_multiple_nodes", text="Multiple Nodes", toggle=True)
        row.prop(settings, "use_camera_rule", text="Camera", toggle=True)

        if settings.use_time_rule:
            time_box = layout.box()
            row = time_box.row(align=True)
            row.prop(settings, "show_time_options", icon='TRIA_DOWN' if settings.show_time_options else 'TRIA_RIGHT', icon_only=True, emboss=False)
            row.label(text=iface_("Time Options:"))
            if settings.show_time_options:
                row = time_box.row()
                row.prop(settings, "use_year")
                row.prop(settings, "use_month")
                row.prop(settings, "use_day")
                row = time_box.row()
                row.prop(settings, "use_hour")
                row.prop(settings, "use_minute")
                row.prop(settings, "use_second")

        if settings.use_random_rule:
            random_box = layout.box()
            row = random_box.row(align=True)
            row.prop(settings, "show_random_options", icon='TRIA_DOWN' if settings.show_random_options else 'TRIA_RIGHT', icon_only=True, emboss=False)
            row.label(text=iface_("Random Options:"))
            if settings.show_random_options:
                random_box.prop(settings, "auto_random", text=iface_("Auto"), toggle=True)

        if settings.use_camera_rule:
            cam_box = layout.box()
            row = cam_box.row(align=True)
            row.prop(settings, "show_camera_info", icon='TRIA_DOWN' if settings.show_camera_info else 'TRIA_RIGHT', icon_only=True, emboss=False)
            row.label(text=iface_("Active Camera Info:"))
            if settings.show_camera_info:
                if bpy.context.scene.camera:
                    cam = bpy.context.scene.camera
                    row = cam_box.row(align=True)
                    row.label(text=f"{iface_('Name')}: {cam.name}")
                    rename_op = row.operator("node.rename_camera", text="", icon='GREASEPENCIL')
                    row = cam_box.row()
                    cam_type = iface_('Orthographic') if cam.data.type == 'ORTHO' else iface_('Perspective')
                    row.label(text=f"{iface_('Type')}: {cam_type}")
                    row = cam_box.row()
                    if cam.data.type == 'ORTHO':
                        row.label(text=f"{iface_('Scale')}: {cam.data.ortho_scale:.1f}")
                    else:
                        row.label(text=f"{iface_('Focal Length')}: {cam.data.lens:.1f}mm")
                else:
                    row = cam_box.row()
                    row.label(text=iface_("No active camera"), icon='ERROR')

        if settings.use_sequence_rule:
            seq_box = layout.box()
            row = seq_box.row(align=True)
            row.prop(settings, "show_sequence_options", icon='TRIA_DOWN' if settings.show_sequence_options else 'TRIA_RIGHT', icon_only=True, emboss=False)
            row.label(text=iface_("Sequence Options:"))
            if settings.show_sequence_options:
                seq_box.prop(settings, "sequence_start")
                seq_box.prop(settings, "sequence_padding")

        if settings.use_suffix_rule:
            suffix_box = layout.box()
            row = suffix_box.row(align=True)
            row.prop(settings, "show_suffix_options", icon='TRIA_DOWN' if settings.show_suffix_options else 'TRIA_RIGHT', icon_only=True, emboss=False)
            row.label(text=iface_("Suffix Options:"))
            if settings.show_suffix_options:
                suffix_box.prop(settings, "custom_prefix")
                suffix_box.prop(settings, "custom_suffix")

        if settings.use_filename_rule:
            filename_box = layout.box()
            row = filename_box.row(align=True)
            row.prop(settings, "show_filename_options", icon='TRIA_DOWN' if settings.show_filename_options else 'TRIA_RIGHT', icon_only=True, emboss=False)
            row.label(text=iface_("Filename Options:"))
            if settings.show_filename_options:
                row = filename_box.row()
                row.prop(settings, "use_interface_name", text=iface_("Use Interface Name"))

        if settings.use_multiple_nodes:
            node_box = layout.box()
            row = node_box.row(align=True)
            row.prop(settings, "show_multiple_nodes_options", icon='TRIA_DOWN' if settings.show_multiple_nodes_options else 'TRIA_RIGHT', icon_only=True, emboss=False)
            row.label(text=iface_("Multiple Nodes:"))
            if settings.show_multiple_nodes_options:
                for item in settings.node_selection:
                    node = context.scene.node_tree.nodes.get(item.node_name) if context.scene.node_tree else None
                    if node:
                        row = node_box.row(align=True)
                        row.prop(item, "select", text="")
                        row.label(text=item.node_name)
                        rename_op = row.operator("node.rename_node", text="", icon='GREASEPENCIL')
                        rename_op.node_name = item.node_name

        blend_filepath = bpy.data.filepath
        if not blend_filepath:
            warning_row = layout.row()
            warning_row.label(text="Warning: Current file is not saved!", icon='ERROR')

@persistent
def render_init_handler(dummy):
    global cached_random_numbers
    try:
        cached_random_numbers.clear()
        for scene in bpy.data.scenes:
            settings = scene.naming_rules_settings
            settings.is_rendering = True
            settings.cached_current_time = datetime.now().strftime("%Y_%m_%d_%H_%M_%S")
            node_tree = scene.node_tree
            if settings.auto_random and settings.use_random_rule:
                if settings.use_multiple_nodes:
                    for item in settings.node_selection:
                        if item.select:
                            node = node_tree.nodes.get(item.node_name)
                            if node and node.type == 'OUTPUT_FILE':
                                for i, file_slot in enumerate(node.file_slots):
                                    key = (node.as_pointer(), i)
                                    if key not in cached_random_numbers:
                                        cached_random_numbers[key] = random.randint(10001, 99999)
                                rename_output_file_paths(node, settings)
                else:
                    node = node_tree.nodes.active
                    if node and node.type == 'OUTPUT_FILE':
                        for i, file_slot in enumerate(node.file_slots):
                            key = (node.as_pointer(), i)
                            if key not in cached_random_numbers:
                                cached_random_numbers[key] = random.randint(10001, 99999)
                        rename_output_file_paths(node, settings)
    except Exception as e:
        print(f"Error in render_init_handler: {e}")

@persistent
def render_complete_handler(scene):
    try:
        for scene in bpy.data.scenes:
            settings = scene.naming_rules_settings
            settings.is_rendering = False
        cached_random_numbers.clear()
    except Exception as e:
        print(f"Error in render_complete_handler: {e}")

@persistent
def render_cancel_handler(scene):
    try:
        for scene in bpy.data.scenes:
            settings = scene.naming_rules_settings
            settings.is_rendering = False
        cached_random_numbers.clear()
    except Exception as e:
        print(f"Error in render_cancel_handler: {e}")

@persistent
def frame_change_post_handler(scene):
    try:
        settings = scene.naming_rules_settings
        node_tree = scene.node_tree
        needs_update = False

        if settings.use_random_rule and settings.auto_random:
            needs_update = True

        if settings.use_frame_rule:
            needs_update = True

        if needs_update and settings.is_rendering:
            current_frame = scene.frame_current
            if settings.last_frame != current_frame:
                settings.last_frame = current_frame
                if settings.use_multiple_nodes:
                    for item in settings.node_selection:
                        if item.select:
                            node = node_tree.nodes.get(item.node_name)
                            if node and node.type == 'OUTPUT_FILE':
                                if settings.auto_random and settings.use_random_rule:
                                    for i, file_slot in enumerate(node.file_slots):
                                        key = (node.as_pointer(), i)
                                        cached_random_numbers[key] = random.randint(10001, 99999)
                                rename_output_file_paths(node, settings, current_frame=current_frame)
                else:
                    node = node_tree.nodes.active
                    if node and node.type == 'OUTPUT_FILE':
                        if settings.auto_random and settings.use_random_rule:
                            for i, file_slot in enumerate(node.file_slots):
                                key = (node.as_pointer(), i)
                                cached_random_numbers[key] = random.randint(10001, 99999)
                        rename_output_file_paths(node, settings, current_frame=current_frame)
    except Exception as e:
        print(f"Error in frame_change_post_handler: {e}")

@persistent
def depsgraph_update_handler(scene, depsgraph):
    """处理节点连接更新"""
    settings = scene.naming_rules_settings
    if not settings.use_filename_rule or not settings.use_interface_name:
        return

    # 检查是否有连接变化
    for update in depsgraph.updates:
        if hasattr(update.id, 'node_tree'):
            node_tree = update.id.node_tree
            if node_tree and (update.is_updated_geometry or update.is_updated_shading):
                # 更新所有选中的文件输出节点
                if settings.use_multiple_nodes:
                    for node in node_tree.nodes:
                        if node.type == 'OUTPUT_FILE':
                            for item in settings.node_selection:
                                if item.node_name == node.name and item.select:
                                    rename_output_file_paths(node, settings)
                                    break
                # 更新活动节点
                elif hasattr(bpy.context, 'active_node'):
                    node = bpy.context.active_node
                    if node and node.type == 'OUTPUT_FILE':
                        rename_output_file_paths(node, settings)

@persistent
def nodes_update_handler(scene):
    """监控节点的添加和删除"""
    settings = scene.naming_rules_settings
    if not settings.use_multiple_nodes:
        return

    # 获取所有节点树
    node_trees = []
    if hasattr(scene, 'node_tree') and scene.node_tree:
        node_trees.append(scene.node_tree)
    for mat in bpy.data.materials:
        if mat.use_nodes and mat.node_tree:
            node_trees.append(mat.node_tree)
    for world in bpy.data.worlds:
        if world.use_nodes and world.node_tree:
            node_trees.append(world.node_tree)

    # 检查新添加的文件输出节点
    for node_tree in node_trees:
        for node in node_tree.nodes:
            if node.type == 'OUTPUT_FILE':
                # 检查节点是否已在选择列表中
                node_found = False
                for item in settings.node_selection:
                    if item.node_name == node.name:
                        node_found = True
                        break
                # 如果是新节点，添加到选择列表
                if not node_found:
                    item = settings.node_selection.add()
                    item.node_name = node.name
                    item.select = True
                    rename_output_file_paths(node, settings)

    # 清理已删除的节点
    i = len(settings.node_selection) - 1
    while i >= 0:
        item = settings.node_selection[i]
        node_exists = False
        for node_tree in node_trees:
            if node_tree.nodes.get(item.node_name):
                node_exists = True
                break
        if not node_exists:
            settings.node_selection.remove(i)
        i -= 1

def register_handler(handler, handler_list):
    if handler not in handler_list:
        handler_list.append(handler)

def unregister_handler(handler, handler_list):
    if handler in handler_list:
        handler_list.remove(handler)

def register():
    global _icons
    _icons = bpy.utils.previews.new()
    bpy.utils.register_class(NodeSelectionItem)
    bpy.utils.register_class(NODE_OT_RenameNode)
    bpy.utils.register_class(NODE_OT_RenameCamera)
    bpy.utils.register_class(NamingRulesSettings)
    bpy.utils.register_class(NODE_PT_CustomOutputPath)
    bpy.types.Scene.naming_rules_settings = bpy.props.PointerProperty(type=NamingRulesSettings)
    
    # Register translations
    bpy.app.translations.register(__name__, translation_dict)
    
    register_handler(render_init_handler, bpy.app.handlers.render_init)
    register_handler(render_complete_handler, bpy.app.handlers.render_complete)
    register_handler(render_cancel_handler, bpy.app.handlers.render_cancel)
    register_handler(frame_change_post_handler, bpy.app.handlers.frame_change_post)
    register_handler(depsgraph_update_handler, bpy.app.handlers.depsgraph_update_post)
    register_handler(nodes_update_handler, bpy.app.handlers.depsgraph_update_post)  # 添加新的处理器

def unregister():
    unregister_handler(render_init_handler, bpy.app.handlers.render_init)
    unregister_handler(render_complete_handler, bpy.app.handlers.render_complete)
    unregister_handler(render_cancel_handler, bpy.app.handlers.render_cancel)
    unregister_handler(frame_change_post_handler, bpy.app.handlers.frame_change_post)
    unregister_handler(depsgraph_update_handler, bpy.app.handlers.depsgraph_update_post)
    unregister_handler(nodes_update_handler, bpy.app.handlers.depsgraph_update_post)  # 注销新的处理器
    
    # Unregister translations
    bpy.app.translations.unregister(__name__)
    
    del bpy.types.Scene.naming_rules_settings
    bpy.utils.unregister_class(NODE_PT_CustomOutputPath)
    bpy.utils.unregister_class(NamingRulesSettings)
    bpy.utils.unregister_class(NODE_OT_RenameCamera)
    bpy.utils.unregister_class(NODE_OT_RenameNode)
    bpy.utils.unregister_class(NodeSelectionItem)

if __name__ == "__main__":
    register()
